#Offline diffraction simulation scripts from pisha2020approximate
#Copyright (C) 2020 Louis Pisha <lpisha@eng.ucsd.edu> / Sonic Arts Research & Development <sonicarts.ucsd.edu> / UC San Diego / The Regents of the University of California 
#See LICENSE.txt for licensing information

# VDaT approximating BTM - Generation of average BTM amplitude responses for
# three cases (Figure 6 in pisha2020approximate). This takes a minute to run.

import random
import math
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.widgets import Button, Slider, CheckButtons, RadioButtons

from common import *
from btm import *
from btm_montecarlo import *

plt.figure(figsize=(6, 5), dpi=100)
magax = plt.axes()

rout = 0.8
bm_sym = BTMMonteCarlo(1.0, -1.0, rout*0.5, rout, 'sym')
bm_asym = BTMMonteCarlo(1.99, -0.01, rout*0.5, rout, 'asym')
bm_outside = BTMMonteCarlo(1.0, -1.0, rout*0.5, rout, 'hole', outside=True)

def redraw():
    magax.clear()
    magax.set_xscale('log')
    magax.set_xlim(20, 25000)
    magax.set_ylim(-50, 20)
    
    magax.plot(bm_sym.get_freqs(), bm_sym.get_avg(), marker='', color='blue', 
        label='Disk in center')
    magax.plot(bm_asym.get_freqs(), bm_asym.get_avg(), marker='', color='green', 
        linestyle='--', label='Disk close to source')
    magax.plot(bm_outside.get_freqs(), bm_outside.get_avg(), marker='', color='orange', 
        linestyle=':', label='Hole (non-shadowed)')
    magax.set_xlabel('Frequency (Hz)')
    magax.set_ylabel('Amplitude response (dB)')
    magax.legend(loc='upper right')
    
    fwlsd = FW_LSD(bm_sym.get_freqs(), bm_sym.get_avg(), bm_asym.get_avg(), True)
    print('asym FW-LSD: {:.1f} dB'.format(fwlsd))

howmanyismany = 300
print('Sym...')
bm_sym.another(howmanyismany)
print('Asym...')
bm_asym.another(howmanyismany)
print('Outside...')
bm_outside.another(howmanyismany)

redraw()
plt.tight_layout()
plt.show()
