#Offline diffraction simulation scripts from pisha2020approximate
#Copyright (C) 2020 Louis Pisha <lpisha@eng.ucsd.edu> / Sonic Arts Research & Development <sonicarts.ucsd.edu> / UC San Diego / The Regents of the University of California 
#See LICENSE.txt for licensing information

# VDaT approximating BTM - Plotting of BTM input data and VDaT amplitude
# response approximations. Used to generate Figure 7 in pisha2020approximate.
# optimizer_gen_data.py must have been run first to generate the data used here.

import math
import numpy as np
import matplotlib.pyplot as plt
import sys, pickle

from common import *
from vdat import VDaT_BTM_Apx

btmapx = VDaT_BTM_Apx()

_, axs = plt.subplots(3, 6, figsize=(24, 12), dpi=100)

for b, axb in enumerate(axs):
    for a, ax in enumerate(axb):
        setup_ax(ax)
        if a % 2 != 0:
            ax.set_ylim(-10, 10)
        if b == 0:
            ax.set_title(('BTM/VDaT vs radius' if a % 2 == 0 else 'Error (BTM - VDaT)')
                + ' for ' + ('outside' if a >= 4 else 'asym' if a >= 2 else 'sym'))
            ax.set_ylabel('20 cm S-K')
        elif b == 1:
            ax.set_ylabel('2 m S-K')
        else:
            ax.set_ylabel('20 m S-K')

if len(sys.argv[1:]) == 0:
    print('This script should be run like "python3 optimizer_run.py out/*.pickle"')
    sys.exit(1)

for f in sys.argv[1:]:
    with open(f, 'rb') as p:
        pkl = pickle.load(p)
        freqs = pkl['freqs']
        color = pkl['color']
        label = pkl['label']
        zi, zd, rin, rout = pkl['zi'], pkl['zd'], pkl['rin'], pkl['rout']
        ztotal = abs(zi) + abs(zd)
        
        # Main VDaT magnitude
        knee = btmapx.knee(ztotal, rout)
        mags_vdat = btmapx.dring_outside(freqs, ztotal, rout) if pkl['outside'] \
            else btmapx.dring_inside(freqs, ztotal, rout)
        mags_vdat = todb(mags_vdat)
        
        # Plots
        b = 2 if '20p0_' in label else 1 if '2p0_' in label else 0
        a = 4 if 'outside' in label else 2 if 'asym' in label else 0
        
        m_lo = sum([pkl['mags'][m] for m in range(3)]) / 3
        
        axs[b][a].plot([knee, knee], [-100, 100], marker='', alpha=0.3, color=color)
        axs[b][a].plot(freqs, pkl['mags'], marker='', color=color, label=label)
        axs[b][a].plot(freqs, mags_vdat, marker='', color=color)
        
        axs[b][a+1].plot(freqs, pkl['mags'] - mags_vdat, marker='', color=color, label='Err '+label)

plt.tight_layout()
plt.show()
