#Offline diffraction simulation scripts from pisha2020approximate
#Copyright (C) 2020 Louis Pisha <lpisha@eng.ucsd.edu> / Sonic Arts Research & Development <sonicarts.ucsd.edu> / UC San Diego / The Regents of the University of California 
#See LICENSE.txt for licensing information

# VDaT approximating BTM - Diffraction path length approximation.
# Generates Figure 10 in pisha2020approximate. Takes a minute.

import sys
import numpy as np
import matplotlib
import matplotlib.pyplot as plt

from math import *
from common import *
from examples3d import *
from xtd import *
from vdat import vdat

c = 345.0
ringdefs = defaultringdefs(9, 64, 44100, c)
vdatparams = defaultvdatparams()
freqs = np.array([1000.0])

verts, tris, edges = halfplane(50.0)

zs = []
reallens = []
vdatlens = []

for i in range(-60, 32):
    z = pow(2.0, i/10.0)
    ps = np.array([-1, 0, -z])
    pk = np.array([1, 0, -z])
    _, reallen = call_xtd(gtd_core, verts, tris, edges, ps, pk, freqs, c, True, True)
    _, _, vdatlen, _ = vdat(verts, tris, edges, ps, pk, ringdefs, freqs, True, vdatparams)
    reallen = reallen[0]
    vdatlen = vdatlen[0]
    print('Final len {:.4f}, real len {:.4f}'.format(vdatlen, reallen))
    zs.append(z)
    reallens.append(reallen-2.0)
    vdatlens.append(vdatlen-2.0)
    
fig, ax = plt.subplots()
ax.set_xscale('log', basex=10)
ax.set_yscale('log', basey=10)
ax.plot(zs, reallens, marker='o', color='blue', label='Real')
ax.plot(zs, vdatlens, marker='.', color='darkorange', label='VDaT')
ax.legend(loc='upper left')
ax.set_xlabel('Parallel (Y) distance from edge (m)')
ax.set_ylabel('Path length minus 2 (m)')
ax.set_title('Edge diffraction vs. VDaT estimated path length')
plt.show()
